package com.datacloudsec.scan.tasks.scheduler;

import com.datacloudsec.scan.common.Config;
import com.datacloudsec.scan.tasks.scheduler.job.IJob;
import com.datacloudsec.scan.tasks.scheduler.time.ITime;
import com.datacloudsec.scan.tasks.scheduler.time.TimeIntervalHour;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;

public class PeriodTask {

   private static PeriodTask instance;
   private ThreadPoolExecutor threadPool;
   private long currTime;
   private Map tasks;
   private Logger LOG;
   private boolean periodExec;


   private PeriodTask() {
      this.threadPool = new ThreadPoolExecutor(10, 10, 0L, TimeUnit.SECONDS, new LinkedBlockingQueue(1000));
      this.currTime = System.currentTimeMillis();
      this.tasks = new HashMap();
      this.LOG = Logger.getLogger(PeriodTask.class);
   }

   public static synchronized PeriodTask getInstance() throws Exception {
      if(instance == null) {
         instance = new PeriodTask();
         instance.periodExec = Config.getInstance().getBoolean("periodExec", false);
         instance.checkSystimeUpdate();
         instance.start();
      }

      return instance;
   }

   public synchronized void addJob(Class jobClass, ITime time, String id, Map params) throws Exception {
      if(this.periodExec) {
         IJob job = (IJob)jobClass.newInstance();
         job.setParams(params);
         PeriodTask.Task task = new PeriodTask.Task(this.threadPool, time, job);
         task.reCalcSecond(this.currTime);
         this.tasks.put(id, task);
      }
   }

   public synchronized void removeJob(String id) throws Exception {
      if(this.periodExec) {
         this.tasks.remove(id);
      }
   }

   private synchronized void refreshTasks() throws Exception {
      this.currTime = System.currentTimeMillis();
      Iterator var2 = this.tasks.values().iterator();

      while(var2.hasNext()) {
         PeriodTask.Task task = (PeriodTask.Task)var2.next();
         task.reCalcSecond(this.currTime);
      }

   }

   private synchronized void execTasks() throws Exception {
      Iterator var2 = this.tasks.values().iterator();

      while(var2.hasNext()) {
         PeriodTask.Task task = (PeriodTask.Task)var2.next();
         task.exec(this.currTime);
      }

   }

   private void start() {
      (new Thread() {
         public void run() {
            while(true) {
               try {
                  Thread.sleep(1000L);
               } catch (Exception var3) {
                  PeriodTask.this.LOG.error("发生错误，定时任务已停止运行", var3);
                  return;
               }

               try {
                  PeriodTask.this.execTasks();
               } catch (Exception var2) {
                  ;
               }
            }
         }
      }).start();
   }

   private void checkSystimeUpdate() {
      (new Thread() {
         public void run() {
            while(true) {
               try {
                  Thread.sleep(1000L);
               } catch (Exception var4) {
                  PeriodTask.this.LOG.error("发生错误，定时任务已停止运行", var4);
                  return;
               }

               PeriodTask.this.currTime = PeriodTask.this.currTime + 1000L;

               try {
                  long sysTime = System.currentTimeMillis();
                  if(Math.abs(sysTime - PeriodTask.this.currTime) >= 5000L) {
                     PeriodTask.this.currTime = sysTime;
                     PeriodTask.this.refreshTasks();
                  }
               } catch (Exception var3) {
                  ;
               }
            }
         }
      }).start();
   }

   public static void main(String[] args) throws Exception {
      HashMap params = new HashMap();
      params.put("name", "name111");
      System.out.println("---------------------start_time: " + (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(new Date()) + "---------------------------");
      getInstance().addJob(PeriodTask.TestJob.class, new TimeIntervalHour(2, 0, 0), "ccc", params);
   }

   private static class Task {

      private ITime time;
      private IJob job;
      private int second;
      private ThreadPoolExecutor threadPool;


      public Task(ThreadPoolExecutor threadPool, ITime time, IJob job) {
         this.threadPool = threadPool;
         this.time = time;
         this.job = job;
      }

      public void exec(long currTime) throws Exception {
         --this.second;
         if(this.second <= 0) {
            this.threadPool.execute(this.job);
            this.second = this.time.calcSecond(currTime);
         }

      }

      public void reCalcSecond(long currTime) throws Exception {
         this.second = this.time.calcSecond(currTime);
      }

      public ITime getTime() {
         return this.time;
      }

      public IJob getJob() {
         return this.job;
      }

      public int getSecond() {
         return this.second;
      }

      public ThreadPoolExecutor getThreadPool() {
         return this.threadPool;
      }

      public void setTime(ITime time) {
         this.time = time;
      }

      public void setJob(IJob job) {
         this.job = job;
      }

      public void setSecond(int second) {
         this.second = second;
      }

      public void setThreadPool(ThreadPoolExecutor threadPool) {
         this.threadPool = threadPool;
      }

      public boolean equals(Object o) {
         if(o == this) {
            return true;
         } else if(!(o instanceof PeriodTask.Task)) {
            return false;
         } else {
            PeriodTask.Task other = (PeriodTask.Task)o;
            if(!other.canEqual(this)) {
               return false;
            } else {
               ITime this$time = this.getTime();
               ITime other$time = other.getTime();
               if(this$time == null) {
                  if(other$time != null) {
                     return false;
                  }
               } else if(!this$time.equals(other$time)) {
                  return false;
               }

               IJob this$job = this.getJob();
               IJob other$job = other.getJob();
               if(this$job == null) {
                  if(other$job != null) {
                     return false;
                  }
               } else if(!this$job.equals(other$job)) {
                  return false;
               }

               if(this.getSecond() != other.getSecond()) {
                  return false;
               } else {
                  ThreadPoolExecutor this$threadPool = this.getThreadPool();
                  ThreadPoolExecutor other$threadPool = other.getThreadPool();
                  if(this$threadPool == null) {
                     if(other$threadPool != null) {
                        return false;
                     }
                  } else if(!this$threadPool.equals(other$threadPool)) {
                     return false;
                  }

                  return true;
               }
            }
         }
      }

      protected boolean canEqual(Object other) {
         return other instanceof PeriodTask.Task;
      }

      public int hashCode() {
         boolean PRIME = true;
         byte result = 1;
         ITime $time = this.getTime();
         int result1 = result * 59 + ($time == null?0:$time.hashCode());
         IJob $job = this.getJob();
         result1 = result1 * 59 + ($job == null?0:$job.hashCode());
         result1 = result1 * 59 + this.getSecond();
         ThreadPoolExecutor $threadPool = this.getThreadPool();
         result1 = result1 * 59 + ($threadPool == null?0:$threadPool.hashCode());
         return result1;
      }

      public String toString() {
         return "PeriodTask.Task(time=" + this.getTime() + ", job=" + this.getJob() + ", second=" + this.getSecond() + ", threadPool=" + this.getThreadPool() + ")";
      }
   }

   public static class TestJob implements IJob {

      private String name;


      public void run() {
         System.out.println(this.name + "：" + (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(new Date()));

         try {
            FileUtils.writeStringToFile(new File("d:/time.txt"), (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(new Date()));
         } catch (IOException var2) {
            var2.printStackTrace();
         }

      }

      public void setParams(Map params) throws Exception {
         this.name = params.get("name").toString();
      }

      public void setName(String name) {
         this.name = name;
      }
   }
}
